TLMPWORK_VERSION=0
TLMPWORK_RELEASE=1
TLMP_LIB=$(RPM_BUILD_ROOT)/usr/lib/tlmp
USR_BIN=$(RPM_BUILD_ROOT)/usr/bin
USR_INCLUDE=$(RPM_BUILD_ROOT)/usr/include/tlmp
LDEVEL=/usr/lib/linuxconf-devel

FLAGS=-funsigned-char -g -O2 -Wall 
EGCS=$(shell test -f /usr/bin/egcs && /bin/echo -n 1)
ifeq ($(EGCS),1)
# RedHat has decided that egcs was for rh6.2 compatibility
# instead of using an environment variable. Very odd
# On rh7, we will use gcc. At some point egcs will be history anyway
RH7=$(shell grep "release 7" /etc/redhat-release 2>/dev/null | wc -l)
RH7_=$(shell expr $(RH7))
ifeq ($(RH7_),1)
GPPLINK=gcc
GPP=gcc
else
GPPLINK=egcs
GPP=egcs
endif
else
GPP=g++
# Using g++ to link pulls libg++ even if the program does not need it
GPPLINK=gcc
endif

all:
ifdef DIRS
	@for i in $(DIRS); do make GCC=$(GCC) GPP=$(GPP) -C $$i $@ || exit 1; done
endif


CFLAGS= -DPUBLIC= -DPRIVATE= -DPROTECTED= 

CPP_DEFINES=-Dexport= -DSTATIC= -DEXPORT= -DPUBLIC= -DPROTECTED= -DPRIVATE= -DVIRTUAL= \
	-Dnear= -Dfar=  -D_GNU_SOURCE -DUNIX -DTLMPWORK_VERSION=$(TLMPWORK_VERSION) \
	-DTLMPWORK_RELEASE=$(TLMPWORK_RELEASE)

.cc.o:
	@echo "    " $<
	@$(GPP) $(CFLAGS) $(FLAGS) $(CPP_DEFINES) \
		-I../framework -I/usr/include/tlmp -I/usr/include/linuxconf -c $< -o $@


.SUFFIXES: .tlcc .cc .obt
.tlcc.o:
	@echo "    " $<
	@tlcc $< `basename $< .tlcc`.cpp
	@$(GPP) $(CFLAGS) $(FLAGS) $(CPP_DEFINES) \
		-I../framework -I/usr/include/tlmp -I/usr/include/linuxconf \
		-c `basename $< .tlcc`.cpp -o $@ || \
		( rm -f `basename $< .tlcc`.cpp ; exit 1)
	@rm -f `basename $< .tlcc`.cpp

.tlcc.obt:
	@echo "    " $<
	@tlcc $< `basename $< .tlcc`.cpp
	@$(GPP) $(CFLAGS) $(FLAGS) $(CPP_DEFINES) -DTEST \
		-I../framework -I/usr/include/tlmp -I/usr/include/linuxconf \
		-c `basename $< .tlcc`.cpp -o $@ || \
		( rm -f `basename $< .tlcc`.cpp ; exit 1)
	@rm -f `basename $< .tlcc`.cpp

lib: $(CURDIR).a

$(CURDIR).a: $(OBJS)
	ar cr $(CURDIR).a $(OBJS)
	ranlib $(CURDIR).a


msg:
	$(LDEVEL)/msgscan $(CURDIR) \
		$(CURDIR).dic $(CURDIR).m EF *.cc *.tlcc

msg.clean:
	$(LDEVEL)/msgclean $(CURDIR).dic
	$(LDEVEL)/msgscan $(CURDIR) \
		$(CURDIR).dic $(CURDIR).m EF *.cc *.tlcc

msg.eng:
	mkdir -p $(TLMP_LIB)/help.eng
	$(LDEVEL)/msgcomp -p./ \
		-p$(TLMP_LIB)/messages/sources/ \
		$(TLMP_LIB)/help.eng/tlmpwork.eng eE \
		../tlmpmail/tlmpmail ../framework/framework ../tledit/tledit \
		../agenda/agenda

msg.fr:
	mkdir -p $(TLMP_LIB)/help.fr
	$(LDEVEL)/msgcomp -p./ \
		-p$(TLMP_LIB)/messages/sources/ \
		$(TLMP_LIB)/help.fr/tlmpwork.fr TFeE \
		../tlmpmail/tlmpmail ../framework/framework ../tledit/tledit \
		../agenda/agenda

ifdef DIRS
clean: $(LOCAL_CLEAN)
	rm -f x *.o *.obt *.bak .bak *~ *.a *.old *.log *.nap *.nar core
	@for i in $(DIRS); do make -C $$i clean  || exit 1; done
else
clean: $(LOCAL_CLEAN)
	rm -f x *.o *.obt *.bak *~ *.a *.old *.log *.nap *.nar
endif

ifdef DIRS
install: $(LOCAL_INSTALL) install_subdirs $(LOCAL_POSTINSTALL)

install_subdirs:
	@echo doing subdirs
	@for i in $(DIRS); do make -C $$i install  || exit 1 ; done
	@echo end subdirs

else
install: $(LOCAL_INSTALL) $(LOCAL_POSTINSTALL)
endif

