#pragma interface
#ifndef TLMPMAIL_H
#define TLMPMAIL_H

#include <sys/types.h>
#ifndef MISC_H
	#include <misc.h>
#endif

class MAIL_MESSAGE_FULL;
class PRIVATE_MESSAGE;

enum CONTENT_TYPE {
	CONTENT_TYPE_TEXT,
	CONTENT_TYPE_MULTIPART,
};

class MAIL_MESSAGE: public ARRAY_OBJ{
	class FOLDER_VIEW *folder;
	int usage;
public:
	bool deleted;
	bool replied;
	bool viewed;
	SSTRINGS attrib;
	SSTRING from;
	SSTRING to;
	SSTRING cc;
	SSTRING email;
	SSTRING reply;
	SSTRING reply_email;
	SSTRING name;
	SSTRING subject;
	SSTRING date;
	SSTRING id;
	SSTRING replyid;
	SSTRING received;
	SSTRING content_type;
	off_t filepos;
	int size;
	/*~PROTOBEG~ MAIL_MESSAGE */
public:
	MAIL_MESSAGE (void);
	void assign (FOLDER_VIEW *_folder);
	void decrusage (void);
private:
	void folder_wakeup (void);
public:
	int getusage (void);
	void incrusage (void);
	int loadmsg (MAIL_MESSAGE_FULL&full);
	CONTENT_TYPE parsetype (SSTRING&boundary);
	void set_deleted (bool val);
	void set_replied (bool val);
	void set_viewed (bool val);
	/*~PROTOEND~ MAIL_MESSAGE */
};

class MAIL_MESSAGE_FULL: public MAIL_MESSAGE{
public:
	SSTRING header;
	SSTRING text;
	SSTRINGS attach;
	SSTRINGS spc_headers;		// Special headers
	SSTRING bcc;
	SSTRING fcc;
	/*~PROTOBEG~ MAIL_MESSAGE_FULL */
	/*~PROTOEND~ MAIL_MESSAGE_FULL */
};

class MAIL_MESSAGES: public ARRAY{
	/*~PROTOBEG~ MAIL_MESSAGES */
public:
	MAIL_MESSAGE *getitem (int no)const;
	/*~PROTOEND~ MAIL_MESSAGES */
};

class FOLDER_VIEW: public ARRAY_OBJ{
protected:
	class FOLDER_VIEW_PRIVATE *priv;
public:
	virtual int loadindex (MAIL_MESSAGES &tb)=0;
	virtual int update (bool clean)=0;
	virtual const char *gettitle ()=0;
	virtual int loadmsg (MAIL_MESSAGE &msg, MAIL_MESSAGE_FULL &full)=0;
	virtual bool is_postponed()=0;
	virtual bool is_shared()=0;
	virtual int getid(SSTRING &id)=0;	// Return a string identifying the
										// folder or virtual folder
	/*~PROTOBEG~ FOLDER_VIEW */
public:
	FOLDER_VIEW (void);
	virtual void decrusage (void);
	virtual PRIVATE_MESSAGE&getmsg (void);
	virtual int getusage (void);
	virtual void incrusage (void);
	virtual ~FOLDER_VIEW (void);
	/*~PROTOEND~ FOLDER_VIEW */
};

class FOLDER_FILE: public FOLDER_VIEW{
	SSTRING path;
	MAIL_MESSAGES tb;
	time_t last;		// Time stamp of the folder
	/*~PROTOBEG~ FOLDER_FILE */
public:
	FOLDER_FILE (const char *_path);
	int getid (SSTRING&id);
	const char *gettitle (void);
	bool is_postponed (void);
	bool is_shared (void);
	int loadindex (MAIL_MESSAGES&idx);
	int loadmsg (MAIL_MESSAGE&msg,
		 MAIL_MESSAGE_FULL&full);
	int update (bool clean);
	/*~PROTOEND~ FOLDER_FILE */
};

enum FILTER_ON {
	FILTER_ON_FROM,			// Match the from
	FILTER_ON_SUBJECT,		// Match the subject
	FILTER_ON_BODY,			// Search in the body
	FILTER_ON_VIEWED,		// The message has been viewed or not
	FILTER_ON_REPLIED,		// The message has been replied or not
	FILTER_ON_DELETED,		// The message has been deleted or not
	FILTER_ON_TODAY,		// The message has been received today
	FILTER_ON_THISWEEK,		// The message has been received this week
	FILTER_ON_THISMONTH,		// The message has been received this month
};

enum FILTER_SCOPE {
	FILTER_SCOPE_NONE,		// Do not check this field
	FILTER_SCOPE_FULL,		// Requires a complete match
	FILTER_SCOPE_PARTIAL	// Requires a partial match
};

class FOLDER_FILTER: public FOLDER_VIEW{
	SSTRING title;		// Title of the folder
	// Search criterion
	SSTRING from;
	SSTRING subject;
	SSTRING body;
	bool is_viewed;
	bool is_deleted;
	bool is_replied;
	int since;			// Message received since or -1
	// Each criterion above is checked or not depending on the bool below
	FILTER_SCOPE check_from;
	FILTER_SCOPE check_subject;
	bool check_body;
	bool check_flag_viewed;
	bool check_flag_deleted;
	bool check_flag_replied;
	
	//
	FOLDER_VIEW *folder;	// Folder on which we perform the search
	/*~PROTOBEG~ FOLDER_FILTER */
public:
	FOLDER_FILTER (FOLDER_VIEW *_folder);
	FOLDER_FILTER (FOLDER_VIEW *_folder,
		 FILTER_ON cmd,
		 const char *_key);
	void decrusage (void);
	int editfilter (void);
	int getid (SSTRING&id);
	PRIVATE_MESSAGE&getmsg (void);
	const char *gettitle (void);
	int getusage (void);
	void incrusage (void);
private:
	void init (FOLDER_VIEW *_folder);
public:
	bool is_postponed (void);
	bool is_shared (void);
	int loadindex (MAIL_MESSAGES&idx);
	int loadmsg (MAIL_MESSAGE&msg,
		 MAIL_MESSAGE_FULL&full);
	void setfrom (const char *_from,
		 FILTER_SCOPE scope);
	void setsubject (const char *_subject,
		 FILTER_SCOPE scope);
	int update (bool clean);
	~FOLDER_FILTER (void);
	/*~PROTOEND~ FOLDER_FILTER */
};




#define _TLMP_folder_edit

struct _F_folder_edit{
	class FOLDER_EDIT_PRIVATE *priv;
	void msg_delete (int no);
	void msg_reply (int no);
	void msg_forward (int no);
	void msg_tag (int no, bool set, const char *tag);
	#define _F_folder_edit_loadindex(nom) void nom loadindex(FOLDER_VIEW *folder, MAIL_MESSAGES &tb)
	virtual _F_folder_edit_loadindex ( )=0;
	#define _F_folder_edit_showmsg(nom) void nom showmsg(int no, MAIL_MESSAGE &msg, FOLDER_VIEW *folder)
	virtual _F_folder_edit_showmsg( )=0;
	#define _F_folder_edit_command(nom) void nom command(int cmd)
	virtual _F_folder_edit_command( )=0;
	#define _F_folder_edit_popup(nom) void nom popup(int no, MAIL_MESSAGE &msg, FOLDER_VIEW *folder)
	virtual _F_folder_edit_popup( );
	#define _F_folder_edit_stats(nom) void nom stats(int curmsg, int nbmsg, int nbnew)
	virtual _F_folder_edit_stats( );
	#define _F_folder_edit_event(nom) void nom event(int no, MAIL_MESSAGE &msg)
	virtual _F_folder_edit_event( );
};
	
#define _TLMP_MAIL_COMPOSE

struct _F_MAIL_COMPOSE{
	#define _F_MAIL_COMPOSE_postpone(nom) void nom postpone (int id, MAIL_MESSAGE_FULL &msg)
	virtual _F_MAIL_COMPOSE_postpone( )=0;
	#define _F_MAIL_COMPOSE_archive(nom) void nom archive (int id, MAIL_MESSAGE_FULL &msg)
	virtual _F_MAIL_COMPOSE_archive( );
	#define _F_MAIL_COMPOSE_send(nom) int nom send (int id, MAIL_MESSAGE_FULL &msg)
	virtual _F_MAIL_COMPOSE_send( )=0;
	#define _F_MAIL_COMPOSE_outbox(nom) int nom outbox (int id, MAIL_MESSAGE_FULL &msg)
	virtual _F_MAIL_COMPOSE_outbox( );
	#define _F_MAIL_COMPOSE_quotereply(nom) void nom quotereply (MAIL_MESSAGE_FULL &msg, SSTRING &txt)
	virtual _F_MAIL_COMPOSE_quotereply( );
	#define _F_MAIL_COMPOSE_quoteforward(nom) void nom quoteforward (MAIL_MESSAGE_FULL &msg, SSTRING &txt)
	virtual _F_MAIL_COMPOSE_quoteforward( );
	#define _F_MAIL_COMPOSE_initnewmsg(nom) void nom initnewmsg (SSTRING &txt)
	virtual _F_MAIL_COMPOSE_initnewmsg( );
	#define _F_MAIL_COMPOSE_addrbook(nom) void nom addrbook (SSTRINGS &emails)
	virtual _F_MAIL_COMPOSE_addrbook( );
	#define _F_MAIL_COMPOSE_folderlist(nom) void nom folderlist (SSTRING &folder)
	virtual _F_MAIL_COMPOSE_folderlist( );
	#define _F_MAIL_COMPOSE_codialog_setup(nom) void * nom codialog_setup (DIALOG &dia, int &butmask)
	virtual _F_MAIL_COMPOSE_codialog_setup( );
	#define _F_MAIL_COMPOSE_codialog_end(nom) void nom codialog_end (void *data)
	virtual _F_MAIL_COMPOSE_codialog_end( );
	#define _F_MAIL_COMPOSE_codialog_buttons(nom) void nom codialog_buttons (void *data, DIALOG &dia, MENU_STATUS code)
	virtual _F_MAIL_COMPOSE_codialog_buttons( );
	#define _F_MAIL_COMPOSE_codialog_validate(nom) int nom codialog_validate (void *data, DIALOG &dia, int &nof)
	virtual _F_MAIL_COMPOSE_codialog_validate( );
};

class MAIL_COMPOSE{
	class MAIL_COMPOSE_PRIVATE *priv;
	_F_MAIL_COMPOSE &c;
public:
	/*~PROTOBEG~ MAIL_COMPOSE */
public:
	MAIL_COMPOSE (_F_MAIL_COMPOSE&_c);
	void compose (SSTRINGS&recipients);
	void compose (void);
	void forward (MAIL_MESSAGE&msg);
private:
	void manage (void);
public:
	void reply (MAIL_MESSAGE&msg);
	void resume (MAIL_MESSAGE&msg);
	void start_framework (void);
	/*~PROTOEND~ MAIL_COMPOSE */
};


#define _TLMP_MAIL_VIEW

struct _F_MAIL_VIEW{
	#define _F_MAIL_VIEW_reply(nom) void nom reply (MAIL_MESSAGE &msg)
	virtual _F_MAIL_VIEW_reply( );
	#define _F_MAIL_VIEW_forward(nom) void nom forward (MAIL_MESSAGE &msg)
	virtual _F_MAIL_VIEW_forward( );
	#define _F_MAIL_VIEW_codialog_setup(nom) int nom codialog_setup (int id, DIALOG &dia)
	virtual _F_MAIL_VIEW_codialog_setup( );
	#define _F_MAIL_VIEW_codialog_end(nom) void nom codialog_end (int id)
	virtual _F_MAIL_VIEW_codialog_end( );
	#define _F_MAIL_VIEW_codialog_buttons(nom) void nom codialog_buttons (int id, DIALOG &dia, MENU_STATUS code)
	virtual _F_MAIL_VIEW_codialog_buttons( );
	#define _F_MAIL_VIEW_codialog_validate(nom) int nom codialog_validate (int id, DIALOG &dia, int &nof)
	virtual _F_MAIL_VIEW_codialog_validate( );
};


class MAIL_VIEW{
	class MAIL_VIEW_PRIVATE *priv;
	_F_MAIL_VIEW &c;
public:
	/*~PROTOBEG~ MAIL_VIEW */
public:
	MAIL_VIEW (_F_MAIL_VIEW&_c);
	void manage (void);
	void view (MAIL_MESSAGE&msg, bool replace);
	/*~PROTOEND~ MAIL_VIEW */
};

class FOLDERS: public ARRAY{
	/*~PROTOBEG~ FOLDERS */
public:
	FOLDER_VIEW *getitem (int no)const;
	/*~PROTOEND~ FOLDERS */
};



#define _TLMP_maillock

struct _F_maillock{
	#define _F_maillock_ok(nom) void nom ok(const char *path)
	virtual _F_maillock_ok( )=0;
	#define _F_maillock_fail(nom) void nom fail(const char *path)
	virtual _F_maillock_fail( );
	#define _F_maillock_create(nom) int nom create(const char *path)
	virtual _F_maillock_create( );
};

struct ATTACH_INFO {
	const char *body;
	SSTRING encoding;
	SSTRING name;
	SSTRING description;
};


#define _TLMP_mime_split

struct _F_mime_split{
	#define _F_mime_split_part(nom) void nom part(int no, ATTACH_INFO &info)
	virtual _F_mime_split_part( );
	#define _F_mime_split_mainmsg(nom) void nom mainmsg(int no, ATTACH_INFO &info)
	virtual _F_mime_split_mainmsg( );
};


class _F_framework;
class PRIVATE_MESSAGE;

#include "tlmpmail.pp"

#endif

